  ui <- fluidPage(
    sidebarLayout(
      sidebarPanel(
        fileInput("occ", "Choose .csv file with occurrence data (occ.csv)",
                  accept = c(
                    "text/csv",
                    "text/comma-separated-values,text/plain",
                    ".csv")
        ),
        fileInput("detCovs", "Choose .csv file with replicate data (detCovs.csv)",
                  accept = c(
                    "text/csv",
                    "text/comma-separated-values,text/plain",
                    ".csv")
        ),
        fileInput("siteCovs", "Choose .csv file with site data (siteCovs.csv)",
                  accept = c(
                    "text/csv",
                    "text/comma-separated-values,text/plain",
                    ".csv")
        ),
        # Input: Choose dataset ----
        selectInput("downloadFormat", "Choose a dataset format:",
                    choices = c("long", "array3d")),#,choices = c("long (.csv file)", "array3d and formatted covariates (.RDS file to import to R)")),
        
        
        # Button
        downloadButton("downloadData", "Download"),
        tags$hr(),
      ),
      mainPanel(
        textOutput("tableDescription"),
        tableOutput("contents1"),
        
      )
    )
  )
  
  server <- function(input, output) {
    
    objs<-reactive({
      occFile <- input$occ
      detFile <- input$detCovs
      siteFile <- input$siteCovs
      
      if (is.null(occFile)|is.null(detFile)|is.null(siteFile))
        return(NULL)
      
      occ<-read.csv(occFile$datapath, header = TRUE)
      
      detCovs<-read.csv(detFile$datapath, header = TRUE)
      
      siteCovs<-read.csv(siteFile$datapath, header = TRUE)
      
      
      # Start #####
      
      
      # Step 2: Create a unique name for each replicate/site  ####
      occ$replicateID<-paste0(occ$siteID,"_",occ$replicate)
      detCovs$replicateID<-paste0(detCovs$siteID,"_",detCovs$replicate)
      
      
      # Check plots in occ that are not in the siteCovs dataset (must return 0)
      missingSites<-occ$siteID[!occ$siteID%in%siteCovs$siteID]
      
    
      # Check replicates in occ that are not in the detCovs dataset (must return 0)
      # Names must perfectly match!!
      occ[!occ$replicateID%in%detCovs$replicateID,]
      
      # If any of the previous rows returned something, stop and fix the data (e.g. misspellings)
      
      # Some replicates and plot might have been surveyed but no species was ever found, these represent true absences and these plots and replicates must be in the siteCovs and detCovs datasets. But be careful to make sure that only plots that have actually been surveyed are present in these sheets (e.g. a plot that was not surveyed because the weather didn't allow the sampling to take place, should not be in these sheets. Similarly, plots that were planned to be surveyed should not be included).
      
      # Check replicates in detCovs that are not in the occ dataset (check if these replicates were actually surveyed but nothing was found).
      detCovs[!detCovs$replicateID%in%occ$replicateID,] # It is ok and common to have something here, but check if these plots were surveyed (even if nothing was found)!
      
      # Check replicates in detCovs that are not in the occ dataset (check if these replicates were actually surveyed but nothing was found).
      siteCovs[!siteCovs$siteID%in%occ$siteID,] # It is ok to return something, but check if these plots were surveyed (even if nothing was found)!
      
      
      # Step 3: Add column with 1's (assuming each row of the dataset represents one observation) ####
      occ$y<-1
      
      # Include zeroes for species that were not observed in a replicate/site (if zeroes are absent from the occ data)
      # Define number of additional species to be included (data augmentation to estimate global richness (gamma diversity). Can be zero if global richness is not to be estimated).
      nFakeSpecies<-0
      spFake<-paste0(rep("spFake_",nFakeSpecies),seq(1,nFakeSpecies,length=nFakeSpecies))
      
      addData<-expand.grid(replicateID=unique(detCovs$replicateID),speciesID=c(unique(occ$speciesID),spFake))
      addData$siteID<-detCovs$siteID[match(addData$replicateID,detCovs$replicateID)]
      addData$replicate<-detCovs$replicate[match(addData$replicateID,detCovs$replicateID)]
      addData$y<-0
      
      merged<-rbind(
        occ[,c("siteID","replicateID","speciesID","y","replicate")],
        addData[,c("siteID","replicateID","speciesID","y","replicate")])
      
      occZero<-aggregate(y~.,data=merged,FUN=max)
      
      # Step 4: Create numeric indices in occ and make sure of match between predictor and response  variables  ####
      detCovs$replicateID<-factor(detCovs$replicateID)
      occZero$replicateID<-factor(occZero$replicateID,levels=levels(detCovs$replicateID))
      
      siteCovs$siteID<-factor(siteCovs$siteID)
      occZero$siteID<-factor(occZero$siteID,levels=levels(siteCovs$siteID))
      
      occZero$replicate.index<-match(occZero$replicateID,detCovs$replicateID)
      occZero$site.index<-match(occZero$siteID,siteCovs$siteID)
      occZero$species.index<-as.numeric(as.factor(occZero$speciesID))
      
      # Export new occurrence data (complete with zeroes and missing species) ####
      #write.csv(occZero,"occNew.csv")
      
      # If want to create a 3d array to use with other models (site,replicate,species)
      # This will create NAs for replicates where no data is available (not surveyed)
      array3d<-tapply(occZero$y,list(occZero$siteID,occZero$replicate,occZero$speciesID),max)
      
      
      # Observation history for species 1
      array3d[,,1]
      
      # Rearrange covariates
      
      # Creates a simple function that facilitates (get mean values when numeric and most common name when categorical)
      meanCat<-function(x){
        if(is.numeric(x)){resu=mean(x)}
        if(!is.numeric(x)){resu=names(sort(-table(x)))[1]}
        resu
      }
      
      # Rearrange observation covariates into a list of variables (same dimensions as a slice of array)
      detCovs.list<-as.list(detCovs)
      obsCovs3d<-lapply(detCovs.list,
                        function(x){tapply(x,list(detCovs$siteID,detCovs$replicate),meanCat)})
      
      # Check one variable
      names(obsCovs3d)
      obsCovs3d$soilDeph
      
      # Rearrange site covariates
      siteCovs3d<-aggregate(siteCovs,list(siteCovs$siteID),meanCat)
      names(siteCovs3d)
      
      
      return(list(occZero=occZero,
                  array3d=list(occ=array3d,
                               obsCovs=obsCovs3d,
                               siteCovs=siteCovs3d)))
    })
    
    
    output$contents1 <- renderTable({
      
      occFile <- input$occ
      detFile <- input$detCovs
      siteFile <- input$siteCovs
      
      if (is.null(occFile)|is.null(detFile)|is.null(siteFile))
        return(NULL)
      objects<-objs()
  
      if(input$downloadFormat=="array3d"){
        resu<-objects$array3d$occ[,,1]
        
      }
      if(input$downloadFormat=="long"){
        nrows<-ifelse(nrow(objects$occZero)>100,100,nrow(objects$occZero))
        resu<-objects$occZero[1:nrows,]
        
      }
      
      return(resu)
      
    }, rownames = TRUE)
    
    
    output$tableDescription <- renderText({
      occFile <- input$occ
      detFile <- input$detCovs
      siteFile <- input$siteCovs
      if (is.null(occFile)|is.null(detFile)|is.null(siteFile))
        return(NULL)
      
      if(input$downloadFormat=="array3d"){
        desc<-"Showing detection history for first species. Other species and covariates are also formated and will be downloaded as a list of objects in a .rds file. Use readRDS in R to import the list of files and $ to extract the elements of the list"}
      if(input$downloadFormat=="long"){
        desc<-"Showing the first 100 rows of the formated long table (each row is a presence/absence record). Download the file to view the complete dataset"
      }
      return(desc)
      
      })
    
    output$downloadData <- downloadHandler(
      filename = function() {
        if(input$downloadFormat=="array3d"){
          name=paste(input$downloadFormat, ".rds", sep = "") }
        if(input$downloadFormat=="long"){
          name=paste(input$downloadFormat, ".csv", sep = "") }
      return(name)
        },
      content = function(file) {
        objects<-objs()
        if(input$downloadFormat=="array3d"){
          saveRDS(objects$array3d,file)
        }
        if(input$downloadFormat=="long"){
          write.csv(objects$occZero,file, row.names = FALSE)
        }
      }
    )
    
    
    
  }
  
  shinyApp(ui, server)